package haifa.shopsim;

/**This is a command generated by a scheduling algorithm to a job shop. It tells
the shop what to do next. Machines are identified using the machine String. This
string may have a numeric character value siginifing the machine. The Job to do
on the current machine is also a String. This string has the unique job id.*/
public class ShopCommand{

	/**The type of command, schedule, quit, etc...*/
	int code;

	/**when code has this value it means that this is a scheduling command.*/
	public final static int CODE_SCHEDULE = 1;

	/**when code has this value it means that the command is to quit the work
	of the job shop.*/
	public final static int CODE_QUIT	= -1;

	public final static int CODE_REST = -10;

	/**The machine that should do this command.*/
	String machine;

	/**The job that should be performed on this machine.
	This may be of the form (r,o,t) route, operation, type*/
	String job;

	public Operation op;

	/**Constructs a shop command.*/
	public ShopCommand(String machine_,String job_, int code_){
		machine = machine_;
		job = job_;
		code = code_;
	}

	public ShopCommand(String machine_,Operation op_, int code_){
		machine = machine_;
		op = op_;
		code = code_;
	}

	public ShopCommand(String machine_,int code_){
		machine = machine_;
		code =code_;
	}

	/**Ask if the command is a quit command.*/
	public boolean isQuit(){
		return code == CODE_QUIT;
	}

	public boolean isRest(){
		return code == CODE_REST;
	}

	public String toString(){
		return "COMMAND: Schedule " + op +" on machine " + machine;
	}
}//END OF CLASS